% Solve L1 SVM problem
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [w b] = l1svm(y,X,rho,verbose);

% if ~issign(X)
%     error('Only +1 and -1 are accepted in dataset');
% end
if ~exist('verbose','var')
    verbose = false;
end
[T N] = size(X);
num_vars = T;
num_conds = 2*N;
num_sparse = 2*N*T;
f = -ones(num_vars,1);
A_i = zeros(num_sparse,1);
A_j = zeros(num_sparse,1);
A_s = zeros(num_sparse,1);
b = zeros(num_conds,1);
Aeq = y(:).';
beq = 0;
lb = zeros(num_vars,1);
ub = ones(num_vars,1)/T;
yX = repmat(y,1,N).*X;
idx_cond = repmat(1:N,T,1);
idx_sparse = 1:N*T;
A_i(idx_sparse) = idx_cond(:);
A_j(idx_sparse) = repmat(1:T,1,N);
A_s(idx_sparse) = yX(:);
b(idx_cond) = rho;
idx_cond = repmat(N+1:2*N,T,1);
idx_sparse = N*T+1:2*N*T;
A_i(idx_sparse) = idx_cond(:);
A_j(idx_sparse) = repmat(1:T,1,N);
A_s(idx_sparse) = -yX(:);
b(idx_cond) = rho;
A = sparse(A_i,A_j,A_s);
if size(A,1) ~= num_conds || size(A,2) ~= num_vars
    error('This shouldn''t happen');
end
options.solver = 1;
options.verbose = verbose;
[x lambda status] = clp([],f,A,b,Aeq,beq,lb,ub,options);
if status ~= 0
    error('Problem while solving linear program');
end
% Note: in CLP, equality constrains come first in 'lambda'
b = lambda(1);
w = -lambda(2:N+1)+lambda(N+2:2*N+1);
