% Learn SVM simultaneously for all players
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [W b] = learn_simul_svm(X,rho,verbose);

if ~issign(X)
    error('Only +1 and -1 are accepted in dataset');
end
if ~exist('verbose','var')
    verbose = false;
end
[T N] = size(X);
num_vars = T*N;
num_conds = 2*N*(N-1) + T;
num_sparse = 2*N*(N-1)*T + T*N;
num_condseq = N;
num_sparseeq = N*T;
f = -ones(num_vars,1);
A_i = zeros(num_sparse,1);
A_j = zeros(num_sparse,1);
A_s = zeros(num_sparse,1);
b = zeros(num_conds,1);
Aeq_i = zeros(num_sparseeq,1);
Aeq_j = zeros(num_sparseeq,1);
Aeq_s = zeros(num_sparseeq,1);
beq = zeros(num_condseq,1);
lb = zeros(num_vars,1);
next_cond = 0;
next_sparse = 0;
for n = 1:N
    idx_n = [1:n-1 n+1:N];
    idx_alpha = ([1:T]-1)*N+n;
    XnXmn = repmat(X(:,n),1,N-1).*X(:,idx_n);
    idx_cond = repmat(next_cond+[1:N-1],T,1);
    idx_sparse = next_sparse+[1:(N-1)*T];
    A_i(idx_sparse) = idx_cond(:);
    A_j(idx_sparse) = repmat(idx_alpha,1,N-1);
    A_s(idx_sparse) = XnXmn(:);
    b(idx_cond) = rho;
    next_cond = next_cond+N-1;
    next_sparse = next_sparse+(N-1)*T;
    idx_cond = repmat(next_cond+[1:N-1],T,1);
    idx_sparse = next_sparse+[1:(N-1)*T];
    A_i(idx_sparse) = idx_cond(:);
    A_j(idx_sparse) = repmat(idx_alpha,1,N-1);
    A_s(idx_sparse) = -XnXmn(:);
    b(idx_cond) = rho;
    next_cond = next_cond+N-1;
    next_sparse = next_sparse+(N-1)*T;
end
for t = 1:T
    idx_alpha = (t-1)*N+[1:N];
    idx_sparse = next_sparse+[1:N];
    A_i(idx_sparse) = repmat(next_cond+1,1,N);
    A_j(idx_sparse) = idx_alpha;
    A_s(idx_sparse) = ones(1,N);
    b(next_cond+1) = 1/T;
    next_cond = next_cond+1;
    next_sparse = next_sparse+N;
end
if next_sparse ~= num_sparse
    error('This shouldn''t happen');
end
A = sparse(A_i,A_j,A_s);
if size(A,1) ~= num_conds || size(A,2) ~= num_vars
    error('This shouldn''t happen');
end
next_cond = 0;
next_sparse = 0;
for n = 1:N
    idx_alpha = ([1:T]-1)*N+n;
    idx_sparse = next_sparse+[1:T];
    Aeq_i(idx_sparse) = repmat(next_cond+1,1,T);
    Aeq_j(idx_sparse) = idx_alpha;
    Aeq_s(idx_sparse) = X(:,n);
    beq(next_cond+1) = 0;
    next_cond = next_cond+1;
    next_sparse = next_sparse+T;
end
if next_sparse ~= num_sparseeq
    error('This shouldn''t happen');
end
Aeq = sparse(Aeq_i,Aeq_j,Aeq_s);
if size(Aeq,1) ~= num_condseq || size(Aeq,2) ~= num_vars
    error('This shouldn''t happen');
end
options.solver = 1;
options.verbose = verbose;
[x lambda status] = clp([],f,A,b,Aeq,beq,lb,[],options);
if status ~= 0
    error('Problem while solving linear program');
end
% Note: in CLP, equality constrains come first in 'lambda'
b = lambda(1:N);
W = zeros(N);
pos = N;
for n = 1:N
    idx = [1:n-1 n+1:N];
    W(n,idx) = -lambda(pos+1:pos+N-1)+lambda(pos+N:pos+2*(N-1));
    pos = pos+2*(N-1);
end

% Correct zero solution to the "simple bias" solution.
if all(W(:)==0) & all(b==0)
    b = 2*(mean(X,1)<0)-1;
    b = b(:);
end
    